@@ -24,6 +24,7 @@ import typing
 import inspect
 import builtins
 import torchplus as tp
+# from pyoverload import overload, override, Tuple, List, Set, params, null, Array, isarray, isoftype, isofsubclass, isint, isdtype, isitertype, isclassmethod
 from pyoverload import *
 from pyctlib import raw_function, return_type_wrapper, touch
 from functools import wraps
@@ -492,10 +493,8 @@ class Tensor(torch.Tensor):
         to_device = cpu
         if device is None and auto_device: to_device = Device
         if device is not None: to_device = device
-        if to_device == cpu:
-            self = torch.Tensor._make_subclass(cls, data, requires_grad)
-        else:
-            self = torch.cuda.Tensor._make_subclass(cls, data, requires_grad)
+        if to_device != data.device: data = data.to(to_device)
+        self = torch.Tensor._make_subclass(cls, data, requires_grad)
         if isinstance(data, cls):
             self.special_from_(data)
         return self
@@ -781,13 +780,34 @@ class Tensor(torch.Tensor):
     @property
     def has_special(self): return self.has_batch or self.has_channel
 
-    def remove_special(self):
+    def remove_special_(self):
         self._batch_dimension = None
         self._channel_dimension = None
+        return self
+
+    def normalize_(self):
+        m, M = self.min(), self.max()
+        if m == M:
+            if M >= 1: return self.one_()
+            if m <= 0: return self.zero_()
+            return self
+        self.sub_(m)
+        self.div_(M-m)
+        return self
+
+    def normalize(self):
+        m, M = self.min(), self.max()
+        if m == M:
+            if M >= 1: return ones_like(self)
+            if m <= 0: return zeros_like(self)
+            return self
+        return (self - m) / (M - m)
 
     def tensor(self): return super()._make_subclass(torch.Tensor, self, self.requires_grad)
     def numpy(self): return super(torch.Tensor, self.cpu().detach()).numpy()
 
+    def one_(self): return self.zero_().add_(1)
+
     @params
     def size(self, *k: [builtins.int, str]):
         if len(k) == 0:
@@ -1136,33 +1156,32 @@ def unsqueeze(tensor: Array.Torch, *dim: int):
 def tensor(data, *, dtype=None, device=None, requires_grad=False, pin_memory=False):
     if device is None and _auto_device is True:
         device = Device
-    if isinstance(data, torch.Tensor) and type(data) is not torch.Tensor:
-        if device == torch.device('cpu'):
-            return torch.Tensor._make_subclass(torch.Tensor, data, data.requires_grad)
-        else:
-            return torch.cuda.Tensor._make_subclass(torch.cuda.Tensor, data, data.requires_grad).to(Device)
-    return torch.tensor(data, dtype=dtype, device=device, requires_grad=requires_grad, pin_memory=pin_memory)
+    if isinstance(data, torch.Tensor):
+        return data.as_subclass(Tensor)
+    return torch.tensor(data, dtype=dtype, device=device, requires_grad=requires_grad, pin_memory=pin_memory).as_subclass(Tensor)
+
+# print(torch_type_list - (torch_type_list - globals()))
+# print(torch_dtype_list - (torch_dtype_list - globals()))
+# print(torch_func_list - (torch_func_list - globals()))
+
+for key in torch_dtype_list:
+    if not (key in __all__ or key in globals()):
+        exec(f"{key} = torch.{key}")
+        __all__.append(key)
+
+for key in torch_type_list:
+    if not (key in __all__ or key in globals()):
+        exec(f"{key} = torch.{key}")
+        __all__.append(key)
 
 for key in dir(torch):
     if key.startswith("_"):
         continue
     if inspect.isclass(eval(f"torch.{key}")):
         continue
-    if key in __all__ or key in globals():
+    if (key in __all__ or key in globals()) and key not in {"typename"}:
         continue
-    if key in torch_dtype_list:
-        # dtypes
-        exec(f"{key} = torch.{key}")
-        __all__.append(key)
     if key in torch_func_list:
         # functions
         exec(f"{key} = torch.{key}")
         __all__.append(key)
-    if key in torch_type_list:
-        exec(f"{key} = torch.{key}")
-        __all__.append(key)
-
-# for key in '''
-# no_grad
-# '''.split():
-#     exec(f"{key} = torch.{key}")
